<?php
/**
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 * @author    Parcelpro
 * @copyright 2020 Parcelpro
 * @license   LICENSE.txt
 */

$autoloadPath = _PS_MODULE_DIR_ . 'parcelpro/vendor/autoload.php';
if (file_exists($autoloadPath)) {
    require_once $autoloadPath;
}

if (!defined('_PS_VERSION_')) {
    exit;
}

use \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButton;
use Parcelpro\Grid\Action\Bulk\PrintLabelBulkAction;

require 'helperpresta.php';

class Parcelpro extends Module
{
    public function __construct()
    {
        $this->name = 'parcelpro';
        $this->tab = 'shipping_logistics';
        $this->version = '1.0.41';
        $this->author = 'Parcel Pro';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '1.5',
            'max' => _PS_VERSION_
        ];
        $this->bootstrap = true;
        $this->displayName = $this->l('parcelpro');
        $this->description = $this->l('parcelproModuleName');
        $this->module_key = 'f877d1bcb9540773c3e3b5422208a65f';

        $tab = array_values(array_filter(Tab::getTabs($this->defaultLanguage()), function ($tab) {
            return $tab['class_name'] === 'AdminParcelproConfig';
        }));

        if (!$tab) {
            $this->createTabs();
        }

        $this->updateHooks();
        $this->updateTables();

        parent::__construct();

        $this->confirmUninstall = $this->l('confirmUninstall');

        if (!Configuration::get('PARCELPRO_NAME')) {
            $this->warning = $this->l('noNameProvided.');
        }

        if (_PS_VERSION_ < '1.5') {
            require(_PS_MODULE_DIR_ . $this->name . '/backward_compatibility/backward.php');
        }
    }

    public function install()
    {
        if (!parent::install() || !Configuration::updateValue('PARCELPRO_NAME', 'parcelpro')) {
            return false;
        }

        $this->createTables();
        $this->registerHooks();
        Configuration::set('PS_LANG_DEFAULT', ['NL', 'EN']);

//        $this->createTabs();

        return true;
    }

    public function uninstall()
    {
        if (!parent::uninstall() || !Configuration::deleteByName('PARCELPRO_NAME')) {
            return false;
        }

        Configuration::deleteByName("PARCELPRO_NAME");
        Configuration::deleteByName("parcelproId");
        Configuration::deleteByName("apiKey");
        Configuration::deleteByName("paymentMethod");

        $this->deleteTabs();
        $this->deleteCarriers();
        $this->deleteTables();
        $this->unregisterHooks();

        return true;
    }

    public function getContent()
    {
        $output = "";

        if (Tools::isSubmit('submit' . $this->name)) {
            $parcelproName = (string)Tools::getValue('PARCELPRO_NAME');
            $parcelproId = (string)Tools::getValue('parcelproId');
            $parcelproApiKey = (string)Tools::getValue('apiKey');
            $paymentMethod = (string)Tools::getValue('paymentMethod');

            if (!$parcelproName && !$parcelproId && !$parcelproApiKey && $paymentMethod ||
                empty($parcelproId && $parcelproApiKey) ||
                !Validate::isGenericName($parcelproId, $parcelproApiKey, $paymentMethod) ||
                !Validate::isAnything($paymentMethod)
            ) {
                $output .= $this->displayError($this->l('invalidConfigValue'));
            } else {
                Configuration::updateValue('PARCELPRO_NAME', $this->name);
                Configuration::updateValue('parcelproId', $parcelproId);
                Configuration::updateValue('apiKey', $parcelproApiKey);
                Configuration::updateValue('paymentMethod', $paymentMethod);
                Configuration::updateValue('shipments', '');
                Configuration::updateValue('triggers_0', (string)Tools::getValue('triggers_0'));
                Configuration::updateValue('triggers_1', (string)Tools::getValue('triggers_1'));
                Configuration::updateValue('triggers_2', (string)Tools::getValue('triggers_2'));

                foreach ($this->triggers() as $trigger) {
                    $deleted = null;
                    if (Tools::getValue('triggers_' . $trigger['id']) == false) {
                        $deleted = date('Y-m-d H:i:s');
                    }

                    $this->api()->triggers(Tools::strtolower($trigger['name']), $trigger['code'], $deleted);
                }

                $output .= $this->displayConfirmation($this->l('settingsUpdated'));
            }
        }

        return $output . $this->displayForm();
    }

    public function displayForm()
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('order_state', 'os');
        $sql->innerJoin('order_state_lang', 'osl', 'os.id_order_state = osl.id_order_state');
        $sql->where('id_lang = ' . (int)$this->defaultLanguage());
        $data = Db::getinstance()->executeS($sql);

        $formFields = [];

        // Init Fields form array
        $formFields[0]['form'] = [
            'legend' => [
                'title' => $this->l('settings'),
            ],
            'input' => [
                [
                    'type' => 'text',
                    'label' => $this->l('userId'),
                    'name' => 'parcelproId',
                    'size' => 20,
                    'required' => true
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('apiKey'),
                    'name' => 'apiKey',
                    'size' => 20,
                    'required' => true
                ],
                [
                    'type' => 'select',
                    'lang' => true,
                    'label' => $this->l('autoAddShipments'),
                    'name' => 'paymentMethod',
                    'required' => true,
                    'options' => [
                        'query' => $data,
                        'id' => 'id_order_state',
                        'name' => 'name'
                    ]
                ],
                [
                    'type' => 'checkbox',
                    'label' => $this->l('triggers'),
                    'name' => 'triggers',
                    'required' => true,
                    'values' => [
                        'query' => $this->triggers(),
                        'id' => 'id',
                        'name' => 'name'
                    ]
                ]
            ],
            'submit' => [
                'title' => $this->l('save'),
                'class' => 'btn btn-default pull-right'
            ],
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&configure=' . $this->name
            . '&tab_module=' . $this->tab . '&module_name=' . $this->name;

        // Language
        $helper->default_form_language = $this->defaultLanguage();
        $helper->allow_employee_form_lang = $this->defaultLanguage();

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('save'),
                'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                    '&token=' . Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('backToList')
            ]
        ];

        // Load current value
        $helper->fields_value['parcelproId'] = Configuration::get('parcelproId');
        $helper->fields_value['apiKey'] = Configuration::get('apiKey');
        $helper->fields_value['paymentMethod'] = Configuration::get('paymentMethod');
        $helper->fields_value['triggers_0'] = Configuration::get('triggers_0');
        $helper->fields_value['triggers_1'] = Configuration::get('triggers_1');
        $helper->fields_value['triggers_2'] = Configuration::get('triggers_2');

        if ($this->api()->validate()) {
            $types = json_decode($this->api()->getType(), true);

            if (isset($types['error'])) {
                return $this->displayError($types['error']) . $helper->generateForm($formFields);
            } else {
                $this->createCarriers($types);
            }
        }

        if (Configuration::get('parcelproId') && Configuration::get('apiKey') && !$this->api()->validate()) {
            return $this->displayError($this->l('userIdOrApiKeyError')) . $helper->generateForm($formFields);
        }

        return $helper->generateForm($formFields);
    }

    public function hookDisplayHeader()
    {
        if (_PS_VERSION_ < 1.7) {
            $this->context->controller->addCSS($this->_path . '/views/css/parcelpro.css');
            $this->context->controller->addJS($this->_path . '/views/js/parcelpro.js');
        } else {
            $this->context->controller->registerStylesheet(
                "module-parcelpro-css",
                $this->_path . 'views/css/parcelpro.css',
                'all'
            );
            $this->context->controller->registerJavascript(
                "module-parcelpro-js",
                $this->_path . 'views/js/parcelpro.js',
                'all'
            );
        }
    }

    public function hookDisplayBackOfficeHeader()
    {
        if ($this->context->controller->controller_name == 'AdminOrders') {
//            $this->context->controller->addJS("/js/jquery/jquery-1.11.0.min.js");
//            $this->context->controller->addJqueryUI('ui.sortable');

            $this->context->controller->addJS($this->_path . "/views/js/admin.js", "all");
            $this->context->controller->addCSS($this->_path . "/views/js/admin.css", "all");
        }
    }

    public function hookDisplayCarrierExtraContent($params)
    {
        $carrierId = $params['carrier']['id'];
        $referenceId = $params['carrier']['id_reference'];

        $query = new DbQuery();
        $query->select('*');
        $query->from('parcelpro_carrier');
        $query->where('id_carrier = ' . (int)$referenceId);

        $select = Db::getInstance()->executeS($query);
        $select = reset($select);

        $servicepoint = false;

        if ($select["servicepoint"]) {
            $servicepoint = true;
        }

        $address = new Address($this->context->cart->id_address_delivery);

        $pickupPoint = new DbQuery();
        $pickupPoint->select('*')
            ->from('cart_pickup_points')
            ->where('cart_id = ' . (int)$this->context->cart->id);
        $pickupPoint = Db::getInstance()->executeS($pickupPoint);

        $pickupPoint = reset($pickupPoint);

        $domain = parse_url($_SERVER['HTTP_REFERER']);

        $url = 'https://login.parcelpro.nl/plugin/afhaalpunt/parcelpro-kiezer.php';
        $url .= '?id=' . pSQL(Configuration::get('parcelproId'));
        $url .= '&postcode=' . $address->postcode;
        $url .= '&adres=' . $address->address1;
        $url .= '&origin=' . $domain['scheme'] . '://' . $domain['host'];
        $url .= '&country=' . Country::getIsoById($address->id_country);
        $url .= '&software=prestashop';
        $url .= '&carrier=' . $select['carrier_name'];

        $this->context->smarty->assign(compact('pickupPoint', 'carrierId', 'servicepoint', 'url'));

        return $this->display(__FILE__, '/checkout.tpl');
    }

    public function hookDisplayAfterCarrier()
    {
        $query = new DbQuery();
        $query->select('id_carrier');
        $query->from('carrier');
        $query->where('active = 1 AND deleted = 0');

        $carriers = Db::getInstance()->executeS($query);

        $carriers = array_map(function ($carrier) {
            return $carrier['id_carrier'];
        }, $carriers);

        $link = new Link();
        $ajax = $link->getModuleLink('parcelpro', 'parcelshop', ['action' => 'postProcess']);

        $pickupPoint = new DbQuery();
        $pickupPoint->select('*')
            ->from('cart_pickup_points')
            ->where('cart_id = ' . (int)$this->context->cart->id);
        $pickupPoint = Db::getInstance()->executeS($pickupPoint);
        $pickupPoint = reset($pickupPoint);

        $addressText = trim(sprintf(
            '%s %s %s',
            $pickupPoint['street'],
            $pickupPoint['house_number'],
            $pickupPoint['house_number_additional']
        ));

        $postal = null;
        $address = new Address($this->context->cart->id_address_delivery);
        if ($address->postcode) {
            $postal = $address->postcode;
        }

        $this->context->smarty->assign([
            'addressId' => $this->context->cart->id_address_delivery,
            'carriers' => json_encode($carriers),
            'ajax' => $ajax,
            'pickupPoint' => $pickupPoint,
            'addressText' => $addressText,
            'selectedLocation' => $this->l('selectedLocation'),
            'postal' => $postal
        ]);

        return $this->display(__FILE__, '/afterCarrier.tpl');
    }

    public function splitStreetAndHousenumber($address)
    {
        $street = "";
        $housenumber = "";
        $additional = "";
        $department = '';

        $combined = trim(sprintf('%s %s', $address->address1, $address->address2));

        if (preg_match('/([\w]+\s?\D+)\s?(\d+)[\s-]?([A-Za-z0-9]*)(.*)/i', $combined, $result)) {
            $street = trim($result[1]);
            $housenumber = $result[2] ? trim($result[2]) : '';
            $additional = $result[3] ? trim($result[3]) : '';
            $department = $result[4] ? trim($result[4]) : '';
        }

        return compact('street', 'housenumber', 'additional', 'department');
    }

    public function hookModuleRoutes()
    {
        $routes = [
            "parcelpro" => [
                "controller" => "parcelshop",
                "rule" => "bestelling/parcelshop",
                "keywords" => [],
                "params" => [
                    "fc" => "module",
                    "module" => "parcelpro"
                ]
            ],
        ];

        return $routes;
    }

    /**
     * 1.7.7
     */
    public function hookActionGetAdminOrderButtons($params)
    {
        $order = new Order(Tools::getValue('id_order'));

        /** @var \PrestaShopBundle\Controller\Admin\Sell\Order\ActionsBarButtonsCollection $bar */
        $bar = $params['actions_bar_buttons_collection'];

        if ($this->api()->validate()) {
            $data = [
                'orderId' => Tools::getValue('id_order'),
                'reference' => $order->reference
            ];

            $shipment = json_decode($this->api()->shipments($data), true);
            $shipment = reset($shipment);

            if ($shipment) {
                $hash = $this->api()->createHash(Configuration::get('parcelproId') . $shipment['Id']);
                $parameters = [
                    'GebruikerId' => Configuration::get('parcelproId'),
                    'ZendingId' => $shipment['Id'],
                    'HmacSha256' => $hash,
                    'PrintPdf' => 1
                ];

                $bar->add(
                    new ActionsBarButton(
                        'btn btn-primary',
                        [
                            'href' => $this->api()->apiUrl() . 'label.php' . '?' .
                                http_build_query($parameters, '', '&')
                        ],
                        $this->l('printLabel')
                    )
                );

                if ($shipment['Afgedrukttijdstip']) {
                    $bar->add(
                        new ActionsBarButton(
                            'btn-success',
                            [
                                'href' => $shipment['TrackingUrl'],
                                'target' => '_blank'
                            ],
                            $this->l('trackAndTrace')
                        )
                    );
                }
            }
        }
    }

    /**
     * 1.7.6
     */
    public function hookDisplayBackOfficeOrderActions()
    {
        $order = new Order(Tools::getValue('id_order'));

        $trackingUrl = null;

        if ($this->api()->validate()) {
            $shipments = json_decode($this->api()->shipments(), true);

            $shipment = array_values(array_filter($shipments, function ($shipment) use ($order) {
                return $shipment['OrderNr'] == $order->id && $shipment['Referentie'] == $order->reference;
            }));

            $shipment = reset($shipment);
            $trackingUrl = $shipment['TrackingUrl'];
        }

        $this->context->smarty->assign([
            "request" => $this->context->link->getAdminLink("AdminParcelpro"),
            "orderId" => Tools::getValue("id_order"),
            'trackingUrl' => $trackingUrl
        ]);

        return $this->context->smarty->fetch(_PS_MODULE_DIR_ . "/parcelpro/views/templates/admin/carriers.tpl");
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        $orderId = $params["id_order"];

        if ($this->api()->validate()) {
            $shipments = json_decode($this->api()->shipments());

            $order = new Order($orderId);

            $filtered = array_filter($shipments, function ($shipment) use ($orderId) {
                return $shipment->OrderNr == $orderId;
            });

            if ($this->checkPaymentStatus($order) && !$filtered) {
                $shipment = json_decode($this->createShipment($order), true);

                $this->api()->postShipment($shipment);
            }
        }
    }

    public function hookActionOrderGridDefinitionModifier(array $params)
    {
        $params['definition']->getBulkActions()->add((new PrintLabelBulkAction('print_labels'))
            ->setName($this->l('printLabel'))
            ->setOptions([
                'submit_route' => 'admin_parcelpro_orders_bulk_label_print'
            ]));
    }

    public function defaultLanguage()
    {
        return Configuration::get('PS_LANG_DEFAULT');
    }

    public function api()
    {
        return new ParcelProAPIWrapper(Configuration::get('parcelproId'), Configuration::get('apiKey'));
    }

    public function carriers()
    {
        return Carrier::getCarriers($this->defaultLanguage(), null, null, null, null, CARRIERS_MODULE_NEED_RANGE);
    }

    public function triggers()
    {
        return [
            [
                'id' => 0,
                'name' => $this->l('printed'),
                'code' => 4
            ],
            [
                'id' => 1,
                'name' => $this->l('inTransit'),
                'code' => 4
            ],
            [
                'id' => 2,
                'name' => $this->l('shipped'),
                'code' => 5
            ]
        ];
    }

    public function checkPaymentStatus($order)
    {
        return $order->current_state == Configuration::get('paymentMethod');
    }

    public function createCarriers($types)
    {
        $currentTypes = array_map(function ($c) {
            return $c['name'];
        }, $this->carriers());

        $filteredTypes = array_values(array_filter($types, function ($type) use ($currentTypes) {
            return !in_array($type['Label'], $currentTypes) && $type["Test"] != "1";
        }));

        foreach ($filteredTypes as $type) {
            $this->createType($type);

            if (in_array($type['Type'], ['Business Parcel', 'UPS Standard'])) {
                if ($type['ServicePoint']) {
                    $this->createType($type, '+ Afhaalpunt');
                }
            }

            if ($type['Type'] == 'DFY') {
                if ($type['ServicePoint']) {
                    $this->createType($type, 'Afhaalpunt');
                }

                if ($type['HandtekeningVoorOntvangst']) {
                    $this->createType($type, 'Handtekening');
                }

                if ($type['NietBijBuren']) {
                    $this->createType($type, 'Niet bij buren');
                }

                if ($type['AvondLevering']) {
                    $this->createType($type, 'Avondlevering');
                }

                if ($type['Buitenland']) {
                    $this->createType($type, 'Buitenland');
                }
            }

            if ($type['Type'] == 'Europlus') {
                if ($type['1100Levering']) {
                    $this->createType($type, '11:00-levering');
                }

                if ($type['ZaterdagLevering']) {
                    $this->createType($type, 'Zaterdaglevering');
                }
            }
        }
    }

    public function deleteCarriers()
    {
        foreach ($this->carriers() as $carrier) {
            if ($carrier['id_carrier'] == $carrier['id_reference']) {
                unlink(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_carrier'] . '.jpg');
            } else {
                if (file_exists(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_carrier'] . '.jpg')) {
                    unlink(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_carrier'] . '.jpg');
                }

                if (file_exists(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_reference'] . '.jpg')) {
                    unlink(_PS_SHIP_IMG_DIR_ . (int)$carrier['id_reference'] . '.jpg');
                }
            }

            Carrier::getCarrierByReference($carrier["id_carrier"])->delete();
        }
    }

    public function createType($type, $extra = null)
    {
        $newType = new Carrier();
        $newType->shipping_method = 1;
        $newType->name = $type['Label'] . ($extra ? ' ' . $extra : '');
        $newType->shipping_external = 0;
        $newType->is_module = 1;
        $newType->need_range = 1;
        $newType->external_module_name = $this->name;
        $newType->delay = [$this->defaultLanguage() => $type["Type"] . ($extra ? ' ' . $extra : '')];
        $newType->setGroups([Group::getCurrent()->id]);
        $newType->active = 0;
        $newType->max_weight = "50";
        $newType->max_height = "50";
        $newType->max_depth = "50";
        $newType->max_width = "50";
        $newType->add();

        copy(
            dirname(__FILE__) . '/views/img/carriers/' . $type['CarrierNaam'] . '.png',
            _PS_SHIP_IMG_DIR_ . '/' . (int)$newType->id . '.jpg'
        );

        foreach (Zone::getZones() as $zone) {
            if ($zone["active"]) {
                $newType->addZone($zone["id_zone"]);
            }
        }

        $rangesWeight = new RangeWeight();
        $rangesWeight->id_carrier = $newType->id;
        $rangesWeight->delimiter1 = 0;
        $rangesWeight->delimiter2 = 100;
        $rangesWeight->add();

        $groups = array_map(function ($group) {
            return $group["id_group"];
        }, Group::getGroups($this->defaultLanguage()));

        $newType->setGroups($groups);

        $servicePoint = (int)$type['ServicePoint'];
        $signatureRequired = null;
        $notToNeighbors = null;
        $eveningDelivery = null;
        $foreign = null;
        $elevenDelivery = null;
        $saturdayDelivery = null;

        if (in_array($type['Type'], ['Business Parcel', 'UPS Standard'])) {
            $servicePoint = null;
            if ($extra == '+ Afhaalpunt') {
                $servicePoint = (int)$type['ServicePoint'];
            }
        }


        if ($type['Type'] == 'DFY') {
            $servicePoint = null;
            if ($extra == 'Afhaalpunt') {
                $servicePoint = (int)$type['ServicePoint'];
            }

            if ($extra == 'Handtekening') {
                $signatureRequired = pSQL($type['HandtekeningVoorOntvangst']);
            }

            if ($extra == 'Niet bij buren') {
                $notToNeighbors = pSQL($type['NietBijBuren']);
            }

            if ($extra == 'Avondlevering') {
                $eveningDelivery = pSQL($type['AvondLevering']);
            }

            if ($extra == 'Buitenland') {
                $foreign = pSQL($type['Buitenland']);
            }
        }

        if ($type['Type'] == 'Europlus') {
            if ($extra == '11:00-levering') {
                $elevenDelivery = pSQL($type['1100Levering']);
            }

            if ($extra == 'Zaterdaglevering') {
                $saturdayDelivery = pSQL($type['ZaterdagLevering']);
            }
        }

        Db::getInstance()->insert('parcelpro_carrier', [
            'id_carrier' => (int)$newType->id,
            'type_id' => (int)$type["Id"],
            'type_name' => pSQL($type["Type"]),
            'type_code' => pSQL($type["Code"]),
            'type_label' => pSQL($type["Label"]),
            'carrier_id' => (int)$type["Carrier_Id"],
            'carrier_name' => pSQL($type["CarrierNaam"]),
            'carrier_label' => pSQL($type["CarrierLabel"]),
            'servicepoint' => $servicePoint,
            'foreign' => $foreign,
            'benelux' => pSQL($type['Benelux']),
            'eu' => pSQL($type['EU']),
            'worldwide' => pSQL($type['Worldwide']),
            'countries' => pSQL($type['Land']),
            'excluded_countries' => null,
            'delivery_point' => pSQL($type['InleverenOpServicepoint']),
            'signature_required' => $signatureRequired,
            'not_to_neighbors' => $notToNeighbors,
            'evening_delivery' => $eveningDelivery,
            'saturday_delivery' => $saturdayDelivery,
            'sunday_delivery' => null,
            'same_day' => null,
            '1100_delivery' => $elevenDelivery,
            'extra_responsibility' => pSQL($type['VerhoogdAansprakelijk']),
            'b2b' => null,
            'b2c' => null,
            'cash_on_delivery' => pSQL($type['Rembours']),
            'return_when_not_home' => null,
            'send_notification' => null,
            'age_check' => null,
            'id_check' => null,
            'toll_required' => pSQL($type['Tolplichtig']),
            'document' => null,
            'mailbox_delivery' => null,
            'return_label' => null,
            'mini_pallet' => pSQL($type['MiniPallet']),
            'pallet' => pSQL($type['Pallet']),
            'collo' => pSQL($type['Collo']),
            'letter' => null,
            'dimensions' => null,
            'deleted' => null,
            'sequence' => null,
            'auto_build' => null,
            'parcelshop_countries' => null,
            'sender_country_nl' => null,
        ]);

        return $newType;
    }

    public function createShipment($order)
    {
        $deliveryAddress = new Address($order->id_address_delivery);
        $splitDeliveryAddress = $this->splitStreetAndHousenumber($deliveryAddress);
        $deliveryCountry = new Country($deliveryAddress->id_country);

        $carrierQuery = new DbQuery();
        $carrierQuery->select('*')
            ->from('carrier', 'c')
            ->rightJoin('parcelpro_carrier', 'pc', 'c.id_reference = pc.id_carrier')
            ->where('c.id_carrier = ' . (int)$order->id_carrier)
            ->limit(1);
        $carrierQuery = Db::getInstance()->executeS($carrierQuery);
        $carrierQuery = reset($carrierQuery);

        $pickupPointPostal = null;

        $checkPickupPoint = new DbQuery();
        $checkPickupPoint->select('*')
            ->from('cart_pickup_points')
            ->where('cart_id = ' . (int)$order->id_cart);

        $pickupPoint = Db::getInstance()->executeS($checkPickupPoint);
        $pickupPoint = reset($pickupPoint);

        if ($pickupPoint) {
            $pickupPointPostal = preg_split(
                '/([A-Za-z]+)/',
                $pickupPoint['postal'],
                -1,
                PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY
            );
        }
        $customer = new Customer($order->id_customer);

        $data = [
            "Carrier" => $carrierQuery["carrier_name"],
            "Type" => $carrierQuery["type_name"],
            'bron' => 'prestashop',
            'origin' => 'http' . ($_SERVER['HTTPS'] == 'on' ? 's' : '') . '://' . $_SERVER['HTTP_HOST'],
            "HandtekeningBijAflevering" => $carrierQuery['signature_required'],
            "NietLeverenBijDeBuren" => $carrierQuery['not_to_neighbors'],
            "DirecteAvondlevering" => $carrierQuery['evening_delivery'],
            "Zaterdaglevering" => $carrierQuery['evening_delivery'],
            "1100levering" => $carrierQuery['1100_delivery'],
            "InleverenOpServicepoint" => null,
            "Extrazeker" => null,
            "Rembours" => null,
            "VerzekerdBedrag" => null,
            "Rekeningnummer" => null,
            "Referentie" => $order->reference,
            "OrderNr" => $order->id,
            "PickupTijdstip" => null,
//            "NaamAfzender" => $pickupAddress->company ?: null,
//            "ContactpersoonAfzender" => $pickupAddress->firstname . " " . $pickupAddress->lastname,
//            "StraatAfzender" => $splitPickupAddress['street'] ?: null,
//            "NummerAfzender" => $splitPickupAddress['housenumber'] ?: null,
//            "ToevoegingAfzender" => isset($splitPickupAddress['additional']) ?
//                $splitPickupAddress['additional'] : null,
//            "PostcodeAfzender" => $pickupAddress->postcode ?: null,
            "PostcodeAfzender" => null,
//            "PlaatsAfzender" => $pickupAddress->city ?: null,
//            "LandAfzender" => $pickupCountry->iso_code ?: null,
//            "EmailAfzender" => Configuration::get('PS_SHOP_EMAIL'),
//            "TelefoonnummerAfzender" => $pickupAddress->phone ?: null,
            "LocationTypeUitreiklocatie" => $pickupPoint['location_type'] ?: null,
            "NameUitreiklocatie" => $pickupPoint['name'] ?: null,
            "StreetUitreiklocatie" => $pickupPoint['street'] ?: null,
            "PostalcodeNumericUitreiklocatie" => $pickupPointPostal[0] ?: null,
            "PostalcodeAlphaUitreiklocatie" => $pickupPointPostal[1] ?: null,
            "CityUitreiklocatie" => $pickupPoint['city'] ?: null,
            "HousenumberUitreiklocatie" => $pickupPoint['house_number'] ?: null,
            "HousenumberAdditionalUitreiklocatie" => $pickupPoint['house_number_additional'] ?: null,
            "ProductCodeUitreiklocatie" => $pickupPoint['pickup_point_id'] ?: null,
            "Naam" => $deliveryAddress->company ?: $deliveryAddress->firstname . " " . $deliveryAddress->lastname,
            "Tav" => $deliveryAddress->company ? $deliveryAddress->firstname . " " . $deliveryAddress->lastname : null,
            'Afdeling' => $splitDeliveryAddress['department'] ?: null,
            "Straat" => $splitDeliveryAddress['street'],
            "Nummer" => $splitDeliveryAddress['housenumber'],
            'Toevoeging' => $splitDeliveryAddress['additional'] ?: null,
            "Postcode" => $deliveryAddress->postcode,
            "Plaats" => $deliveryAddress->city,
            "Land" => $deliveryCountry->iso_code,
            "Email" => $customer->email ?: null,
            "Telefoonnummer" => $deliveryAddress->phone,
            "AantalPakketten" => 1,
            "Gewicht" => $order->getTotalWeight() ?: 1,
            "Opmerking" => $order->getFirstMessage()
        ];

        return json_encode($data);
    }

    public function createTables()
    {
        $sql = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "parcelpro_carrier` (
			`id` int(10) NOT NULL AUTO_INCREMENT,
			`id_carrier` int(10) NOT NULL,
			`type_id` int(10) NOT NULL,
			`type_name` varchar(255) NOT NULL,
			`type_code` varchar(255) NOT NULL,
			`type_label` varchar(255) NOT NULL,
			`carrier_id` int(10) NOT NULL,
			`carrier_name` varchar(255) NOT NULL,
			`carrier_label` varchar(255),
			`carrier_customer_number` varchar(255),
			`servicepoint` int(1),
			primary key (id)
		) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8;";

        $sql .= "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "cart_pickup_points` (
            `cart_id` int(10) NOT NULL,
            `pickup_point_id` int(10) NOT NULL,
            `location_type` varchar(255) NOT NULL,
            `name` varchar(255) NOT NULL,
            `street` varchar(255) NOT NULL,
            `house_number` int(10),
            `house_number_additional` varchar(10),
            `postal` varchar(6),
            `city` varchar(255)
        );";

        Db::getInstance()->execute($sql);
    }

    public function updateTables()
    {
        $checkTable = "SHOW TABLES LIKE '" . _DB_PREFIX_ . "parcelpro_carrier';";

        if (Db::getInstance()->executeS($checkTable)) {
            $checkIfExists = "SHOW COLUMNS FROM `" . _DB_PREFIX_ . "parcelpro_carrier` LIKE 'foreign';";
            $check = Db::getInstance()->executeS($checkIfExists);

            if (!$check) {
                $sql = "ALTER TABLE `" . _DB_PREFIX_ . "parcelpro_carrier` ADD COLUMN(
                    `foreign` bit(1),
                    `benelux` bit(1),
                    `eu` bit(1),
                    `worldwide` bit(1),
                    `countries` varchar(255),
                    `excluded_countries` varchar(255),
                    `delivery_point` bit(1),
                    `signature_required` bit(1),
                    `not_to_neighbors` bit(1),
                    `evening_delivery` bit(1),
                    `saturday_delivery` bit(1),
                    `sunday_delivery` bit(1),
                    `same_day` bit(1),
                    `1100_delivery` bit(1),
                    `extra_responsibility` bit(1),
                    `b2b` bit(1),
                    `b2c` bit(1),
                    `cash_on_delivery` bit(1),
                    `return_when_not_home` bit(1),
                    `send_notification` bit(1),
                    `age_check` bit(1),
                    `id_check` bit(1),
                    `toll_required` bit(1),
                    `document` bit(1),
                    `mailbox_delivery` bit(1),
                    `return_label` bit(1),
                    `mini_pallet` bit(1),
                    `pallet` bit(1),
                    `collo` bit(1),
                    `letter` bit(1),
                    `dimensions` bit(1),
                    `deleted` bit(1),
                    `sequence` bit(1),
                    `auto_build` bit(1),
                    `parcelshop_countries` bit(1),
                    `sender_country_nl` bit(1));";

                Db::getInstance()->execute($sql);
            }
        }
    }

    public function deleteTables()
    {
        $sql = "DROP TABLE IF EXISTS `" . _DB_PREFIX_ . "parcelpro_carrier`;";
        $sql .= "DROP TABLE IF EXISTS `" . _DB_PREFIX_ . "cart_pickup_points`;";

        Db::getInstance()->execute($sql);
    }

    public function createTabs()
    {
        $tab = new Tab();
        $tab->name = [];

        foreach (Language::getLanguages(true) as $language) {
            $tab->name[$language['id_lang']] = $this->l('parcelpro');
        }

        $tab->active = 1;
        $tab->class_name = 'AdminParcelproConfig';
        $tab->id_parent = (int)Tab::getIdFromClassName('IMPROVE');
        $tab->module = $this->name;
        $tab->add();
    }

    public function deleteTabs()
    {
        $tab = Tab::getInstanceFromClassName('AdminParcelproConfig');
        $tab = new Tab($tab->id);
        $tab->delete();
    }

    public function registerHooks()
    {
        $this->registerHook("displayHeader");
        $this->registerHook("displayBackOfficeHeader");
        if (_PS_VERSION_ >= '1.7.7') {
            $this->registerHook("actionGetAdminOrderButtons");
        } else {
            $this->registerHook("displayBackOfficeOrderActions");
        }
        $this->registerHook("displayAfterCarrier");
        $this->registerHook("displayCarrierExtraContent");
        $this->registerHook("moduleRoutes");
        $this->registerHook("actionOrderStatusPostUpdate");
        $this->registerHook('actionOrderGridDefinitionModifier');
    }

    public function updateHooks()
    {
        $moduleId = (int)Module::getModuleIdByName($this->name);
        $hookList = ['actionOrderGridDefinitionModifier', 'displayAfterCarrier'];

        foreach ($hookList as $hookName) {
            $hookId = (int)Hook::getIdByName($hookName);
            if (!Hook::getModulesFromHook($hookId, $moduleId)) {
                DB::getInstance()->insert('hook_module', [
                    'id_module' => $moduleId,
                    'id_shop' => (int)Shop::getContext(),
                    'id_hook' => $hookId
                ]);
            }
        }
    }

    public function unregisterHooks()
    {
        $this->unregisterHook("displayHeader");
        $this->unregisterHook("displayBackOfficeHeader");
        if (_PS_VERSION_ >= '1.7.7') {
            $this->unregisterHook("actionGetAdminOrderButtons");
        } else {
            $this->unregisterHook("displayBackOfficeOrderActions");
        }
        $this->unregisterHook("displayAfterCarrier");
        $this->unregisterHook("displayCarrierExtraContent");
        $this->unregisterHook("moduleRoutes");
        $this->unregisterHook("actionOrderStatusPostUpdate");
        $this->unregisterHook('actionOrderGridDefinitionModifier');
    }
}
